from lkb.lkb_base_class import Concept, Conceptual_Relation, Term
from lkb.neo4j_lkb import Neo4J_Lexical_Knowledge_Base
import pytest
from lkb.basic_lkb import Basic_Lexical_Knowledge_Base
from lkb.rdf_lkb import RDF_Lexical_Knowledge_Base
from lkb.wordnet_lkb import WordNet_Lexical_Knowledge_Base


def pytest_generate_tests(metafunc):
    if "lkb" in metafunc.fixturenames:
        metafunc.parametrize("lkb", ["basic", "rdf", "wordnet", "neo4j"], indirect=True)


@pytest.fixture(scope="module")
def lkb(request):
    if request.param == "basic":
        return Basic_Lexical_Knowledge_Base.read_json("test_data/small_example.json")
    elif request.param == "rdf":
        return RDF_Lexical_Knowledge_Base.read_json("test_data/small_example.json")
    elif request.param == "wordnet":
        return WordNet_Lexical_Knowledge_Base.read_json("test_data/small_example.json")
    elif request.param == "neo4j":
        Neo4J_Lexical_Knowledge_Base.delete_db()
        Neo4J_Lexical_Knowledge_Base.read_json("test_data/small_example.json")
        return Neo4J_Lexical_Knowledge_Base.connect(concept_type=Concept)


def test_concept_getting(lkb):
    c2_concept_dict = {
        "id": "C2",
    }
    assert lkb.get_concept("C2") == Concept(**c2_concept_dict)


def test_term_getting(lkb):
    t1_term_dict = {"id": "T1", "string": "Test Term 1"}
    assert lkb.get_term("T1") == Term(**t1_term_dict)


def test_conceptual_relation_getting(lkb):
    cr1_dict = {"id": "R1", "string": "Test Relation 1"}
    assert lkb.get_relation("R1") == Conceptual_Relation(**cr1_dict)


def test_outward_conceptual_edge_getting(lkb):
    assert [
        (rel.id, concept.id)
        for rel, concept in sorted(
            list(lkb.get_outward_edges("C1")),
            key=(lambda tup: (tup[0].id, tup[1].id)),
        )
    ] == [("R1", "C2"), ("R2", "C3")]


def test_inward_conceptual_edge_getting(lkb):
    assert [
        (rel.id, concept.id)
        for rel, concept in sorted(
            list(lkb.get_inward_edges("C2")),
            key=(lambda tup: (tup[0].id, tup[1].id)),
        )
    ] == [("R1", "C1"), ("R1", "C3")]


def test_synonym_getting(lkb):
    assert sorted([term.id for term in lkb.get_terms_from_concept_id("C3")]) == [
        "T2",
        "T3",
    ]


def test_polyseme_getting(lkb):
    assert sorted([concept.id for concept in lkb.get_concepts_from_term_id("T1")]) == [
        "C1",
        "C2",
    ]
